/*-----------------------------------------------------------------------------+
|       File:       seriell_labview1.c                                         |
|       Autor:      Bubbers                                                    |
|       Klasse:     TGJ1                                                       |
|       Controller: AT89C5131 (Atmel)                                          | 
|       Datum:      16.11.10                                                   |
|       Version:    1.0                                                        |
+------------------------------------------------------------------------------+
|  Beschreibung:                                                               |	
|    Controller schickt bei Tastendruck P3_3 (blau) das mit den 8 Schaltern    |
|    an P1 eingestellte Byte mit 9600 Bit/s ber die serielle Schnittstelle    | 
|    Dieses Zeichen wird auch an P0 mit 8 LEDs dargestellt.                    |
|    Ein empfangenes Byte wird an die 8 LEDs von P2 ausgegeben                 |
+-----------------------------------------------------------------------------*/

#include <at89c5131.h>           // Registeradressen fuer Atmel AT89C5131
 
sfr  at P2   AnzeigeEmpfang;     // 8 LEDs 
sfr  at P0   AnzeigeSenden; 
sfr  at P1   Sendebyte;
sbit at P3_3 Sendetaster;        // Blauer Taster    = Byte an 8 Schaltern senden

#define reloadBRL 217             // Reloadwert des Baudratengenerators 217 fr 9600 Bit/s
                                  // BRL = 256- ( 2^SMOD1 * Fper) / ( 6^(1-SPD) * 32 * Baudrate) 
										    // Fper = Fquarz/2 = 6MHz wenn kein x2-Mode (bei X2-Mode Fper = 12 MHz)

void entprellung (void) //----------- Zeitverzgerung zur Entprellung -----------------------------------
{  unsigned char c;
   for (c=255;c!=0;c--);
}
void main (void) //------------------ Hauptprogramm -----------------------------------------------------
{
   AnzeigeEmpfang = 0;                 // Anzeigen aus
   AnzeigeSenden = 0;                  //
                                       // Initialisierung serielle Schnittstelle
   PCON = PCON | 0b10000000;           // SMOD=1 bei 9600 Baud
	BRL  = reloadBRL;                   // Reloadwert fr int. Baudratengenerator
	BDRCON = 0b00011110;                // int. Baudraten-Generator fr Senden und Empfangen, SPD=1
	SCON   = 0b01010000;                // 8-Bit-UART, Empfang zulassen
	ES     = 0;                         // kein serieller Interrupt
   RI     = 0;                         // Empfangsbit lschen: kein Zeichen empfangen
 	TI     = 1;                         // Sendebit setzen: letztes Byte wurde gesendet
 	
 	while (1)                           // Endlosschleife Hauptprogramm
 	{
 		if (RI)                          // Wenn Zeichen empfangen
 			{AnzeigeEmpfang = SBUF;       // abholen und anzeigen
 			RI = 0;                       // und fr neuen Empfang vorbereiten (Empfangsflag lschen)
 			}
 		else if (Sendetaster)            // Wenn Sendertaster gedrckt
 				{while (!TI);              // warten falls altes Byte noch nicht fertig gesendet
 				AnzeigeSenden = Sendebyte; // Sendebyte dual anzeigen
 				SBUF = Sendebyte;          // und senden
 				TI = 0;                    // Sendeflag lschen
 				entprellung();
 				while (Sendetaster);       // warten bis Taster losgelassen
 				entprellung();
 				} // Ende Senden

	} // Ende Hauptprogrammschleife
} // Ende Hauptprogramm	  	
